      subroutine fdefor(depth,al,aw,dip,uh,uv,un,xr,yr,dis,tlt,str)         
c--------------------------------------
c     subroutine compute deformation for a finite fault,
c     by manipulating calling variables into
c     Chinnery's (double bar) notation ||.  See Okada
c     page 1143.  This routine offsets the variables 
c     across the finite fault surface and then calls
c     the calculating routine (static.f) a total of 4 times.
c     calling parameters:
c        depth:     depth of center of fault surface
c           al:     fault length
c           aw:     fault width
c          dip:     fault dip angle measured at depth reletive
c                   to horizontal
c           uh:     horizontal slip of fault
c           uv:     vertical slip of fault
c           un:     normal slip of fault
c           xr:     receiver location along fault, relative to
c                   center point
c           yr:     receiver location perpendicular
c          dis:     3 element array where dislocations will be
c                   returned. dis(1) is x displacement, dis(2)
c                   is y and dis(3) is z with z positive up.
c          str:     3 element array where strain components are
c                   returned. str(1) is dux/dx, str(2) is
c                   duy/dy, str(3) is (duy/dx + dux/dy)/2
c          tlt:     3 element array where tile components are
c                   returned. tlt(1) is -duz/dx and tlt(2) is -duz/dy
c                   (to match tilt convention that + value is down
c                   in the direction indicated) tlt(3) is the rotation
c                   around a horizontal axis
c
c     All calling parameters are single precision reals.
c
c     program by: Y. Okada     February 1985
c     reworked by: H. Johnson  July 1988
c                  D  Agnew    August 2005 (tlt(3) added)
c--------------------------------------
      implicit double precision (a-h,o-z)
      real depth, dip, al, aw, pr, xr, yr, uh, uv, un, dis, str, tlt
      integer j, k, sign
      dimension dis(*), str(*), tlt(*), tmp1(3), tmp2(4),tmp3(2),
     1 dtmp(3),stmp(4),ttmp(2)
c  common block elascn holds the poisson's ratio, which may be set externally,
c  though not with a data statement
      common/elascn/pr
      data d0, d1, f0, f1, eps/0.0, 1.0, 0.d0, 1.d0, 1.d-3/
      rad=6.283185307179586d0/360.d0
c   compute  alp, a material property parameter made up of
c   Lame's constants lambda and mu: alp = mu / (lambda + mu)
      alp = 1. - 2*pr
      sd=dsin(dip*rad)
      cd=dcos(dip*rad)
      if(dabs(cd) .le. eps) then
        cd=f0
        if(sd .ge. f0) then
          sd=f1
        else sd=-f1
        endif
      endif
c  reset depth and coordinates to make origin the corner of the fault,
c  as used by Okada
      dep = depth + (aw/2.)*sd
      x   = xr    + al/2.
      y   = yr    + (aw/2.)*cd
c  proceed with the Okada computation
      p=y*cd + dep*sd
      q=y*sd - dep*cd
      do 1 i=1,2
        dis(i)=d0
        dis(i+1)=d0
        str(i)=d0
        str(i+1)=d0
        tlt(i)=d0
        tlt(i+1)=d0
        dtmp(i)=f0
        dtmp(i+1)=f0
        stmp(i)=f0
        stmp(i+2)=f0
        ttmp(i)=f0
        tmp1(i)=f0
        tmp1(i+1)=f0
        tmp2(i)=f0
        tmp2(i+2)=f0
        tmp3(i)=f0
1     continue
      do 2 k=1,2
        if(k .eq. 1) then
          eta=p
        else
          eta=p-aw
        endif
        do 3 j=1,2
          if(j .eq. 1) then
            xi=dble(x)
          else
            xi=dble(x)-dble(al)
          endif
          if(j+k .eq. 3) then
            sign=-d1
          else
            sign=d1
          endif
          call static(alp,xi,eta,q,sd,cd,dble(uh),
     :                dble(uv),dble(un),tmp1,tmp2,tmp3)
          do 4 i=1,2
            dtmp(i) = dtmp(i) + sign*tmp1(i)
            stmp(i) = stmp(i) + sign*tmp2(i)
            ttmp(i) = ttmp(i) + sign*tmp3(i)
4         continue
          dtmp(3) = dtmp(3) + sign*tmp1(3)
          stmp(3) = stmp(3) + sign*tmp2(3)
          stmp(4) = stmp(4) + sign*tmp2(4)
3       continue
2     continue
c   find tensor strain, and change sign of tilt
      str(1)=stmp(1)
      str(2)=stmp(4)
      str(3)=(stmp(2)+stmp(3))/2
      do 5 i=1,2
         dis(i) = dtmp(i)
         dis(i+1) = dtmp(i+1)
         tlt(i) = -ttmp(i)
 5    continue
      tlt(3)=(stmp(2)-stmp(3))/2
      return
      end
      subroutine static(alp,xi,eta,q,sd,cd,uh,uv,un,dis,str,tlt)
c--------------------------------------
c     Subroutine to actually compute the dislocation
c     components outlined by equations in Okada on pages
c     1144 to 1148.  Calling parameters are the same
c     as for routine sfault.f except:
c
c          xi:     The offset point on the fault surface
c                  in the length direction.
c                  This takes on values at the four corners
c                  of the finite size fault surface.
c         eta:     Same as above but in width direction.
c
c     program by: Y. Okada     February 1985
c     reworked by: H. Johnson  July 1988
c--------------------------------------
      implicit double precision (a-h,o-z)
      dimension dis(3), str(4), tlt(2)
      data f0,f1,f2/ 0.d0, 1.d0, 2.d0/
      pi2=6.283185307179586d0
      xi2=xi*xi
      eta2=eta*eta
      q2=q*q
      r2=xi2+eta2+q2
      r=dsqrt(r2)
      r3=r*r2
      d=eta*sd-q*cd
      y=eta*cd+q*sd
      reta=r+eta
      if(reta .lt. f0) reta=f0
      rd=r+d
      rrd=f1/(r*rd)
      if(q .eq. f0) then
        tt=f0
      else
        tt=datan(xi*eta/(q*r))
      endif
      if(reta .eq. f0) then
        re=f0
        dle=-dlog(r-eta)
      else
        re=f1/reta
        dle=dlog(reta)
      endif
      rrx=f1/(r*(r+xi))
      rre=re/r
      axi=(f2*r+xi)*rrx*rrx/r
      aeta=(f2*r+eta)*rre*rre/r
      if(cd .ne. f0) then
        td=sd/cd
        x=dsqrt(xi2+q2)
        if(xi .eq. f0) then
          a5=f0
        else
          a5=alp*f2/cd*datan((eta*(x+q*cd)+x*(r+x)*sd)/(xi*(r+x)*cd))
        endif
        a4=alp/cd*(dlog(rd)-sd*dle)
        a3=alp*(y/rd/cd-dle)+td*a4
        a1=-alp/cd*xi/rd-td*a5
        c1=alp/cd*xi*(rrd-sd*rre)
        c3=alp/cd*(q*rre-y*rrd)
        b1=alp/cd*(xi2*rrd-f1)/rd-td*c3
        b2=alp/cd*xi*y*rrd/rd-td*c1
      else
        rd2=rd*rd
        a1=-alp/f2*xi*q/rd2
        a3=alp/f2*(eta/rd+y*q/rd2-dle)
        a4=-alp*q/rd
        a5=-alp*xi*sd/rd
        b1=alp/f2*q/rd2*(f2*xi2*rrd-f1)
        b2=alp/f2*xi*sd/rd2*(f2*q2*rrd-f1)
        c1=alp*xi*q*rrd/rd
        c3=alp*sd/rd*(xi2*rrd-f1)
      endif
      a2=-alp*dle-a3
      b3=-alp*xi*rre-b2
      b4=-alp*(cd/r+q*sd*rre)-b1
      c2=alp*(-sd/r+q*cd*rre)-c3
      do 1 i=1,2
        dis(i)=f0
        dis(i+1)=f0
        str(i)=f0
        str(i+2)=f0
        tlt(i)=f0
1     continue
      if(uh .ne. f0) then
        u1=uh/pi2
        req=rre*q
        dis(1)=dis(1)-u1*(req*xi+tt+a1*sd)
        dis(2)=dis(2)-u1*(req*y+q*cd*re+a2*sd)
        dis(3)=dis(3)-u1*(req*d+q*sd*re+a4*sd)
        str(1)=str(1)+u1*(xi2*q*aeta-b1*sd)
        str(2)=str(2)+u1*(xi2*xi*(d/(eta2+q2)/r3-aeta*sd)-b2*sd)
        str(3)=str(3)+u1*(xi*q/r3*cd+(xi*q2*aeta-b2)*sd)
        str(4)=str(4)+u1*(y*q/r3*cd+(q*sd*(q2*aeta-f2*rre)
     :      -(xi2+eta2)/r3*cd-b4)*sd)
        tlt(1)=tlt(1)+u1*(-xi*q2*aeta*cd+(xi*q/r3-c1)*sd)
        tlt(2)=tlt(2)+u1*(d*q/r3*cd+(xi2*q*aeta*cd-sd/r+y*q/r3-c2)*sd)
      endif
      if(uv .ne. f0) then
        u1=uv/pi2
        sdcd=sd*cd
        dis(1)=dis(1)-u1*(q/r-a3*sdcd)
        dis(2)=dis(2)-u1*(y*q*rrx+cd*tt-a1*sdcd)
        dis(3)=dis(3)-u1*(d*q*rrx+sd*tt-a5*sdcd)
        str(1)=str(1)+u1*(xi*q/r3+b3*sdcd)
        str(2)=str(2)+u1*(y*q/r3-sd/r+b1*sdcd)
        str(3)=str(3)+u1*(y*q/r3+q*cd*rre+b1*sdcd)
        str(4)=str(4)+u1*(y*y*q*axi-(f2*y*rrx+xi*cd*rre)*sd+b2*sdcd)
        tlt(1)=tlt(1)+u1*(d*q/r3+q*sd*rre+c3*sdcd)
        tlt(2)=tlt(2)+u1*(y*d*q*axi-(f2*d*rrx+xi*sd*rre)*sd+c1*sdcd)
      endif
      if(un .ne. f0) then
        u1=un/pi2
        sdsd=sd*sd
        dis(1)=dis(1)+u1*(q2*rre-a3*sdsd)
        dis(2)=dis(2)+u1*(-d*q*rrx-sd*(xi*q*rre-tt)-a1*sdsd)
        dis(3)=dis(3)+u1*(y*q*rrx+cd*(xi*q*rre-tt)-a5*sdsd)
        str(1)=str(1)-u1*(xi*q2*aeta+b3*sdsd)
        str(2)=str(2)-u1*(-d*q/r3-xi2*q*aeta*sd+b1*sdsd)
        str(3)=str(3)-u1*(q2*(cd/r3+q*aeta*sd)+b1*sdsd)
        str(4)=str(4)-u1*((y*cd-d*sd)*q2*axi-f2*q*sd*cd*rrx
     :      -(xi*q2*aeta-b2)*sdsd)
        tlt(1)=tlt(1)-u1*(q2*(sd/r3-q*aeta*cd)+c3*sdsd)
        tlt(2)=tlt(2)-u1*((y*sd+d*cd)*q2*axi+xi*q2*aeta*sd*cd
     :      -(f2*q*rrx-c1)*sdsd)
      endif
      return
      end
