      FUNCTION PCROSP(X,NN,NK,KOFSET,KMAX,M,SP,SCRACH,CF,KTAPR,IE,NEWK)
C
C
C TITLE - PCROSP = PART CROSS-SPECTRUM
C     COMPUTES ONE SECTION OF CROSS-SPECTRAL MATRIX
C
C
C                     ---ABSTRACT---
C
C              PCROSP  IS USED TO COMPUTE THE CROSS POWER SPECTRA OF
C              ALL PAIRS OF COMPONENT SERIES STORED IN AN ARRAY
C              DIMENSIONED  X(NC,N).  THE SERIES MAY BE DIVIDED INTO
C              SECTIONS OF LENGTH  N  WHERE  N  IS A POWER OF TWO AND
C              PCROSP  CALLED FOR EACH SECTION.  THE MEAN OF EACH
C              COMPONENT OF  X  IS COMPUTED AND SUBTRACTED FROM EACH
C              TERM.  THEN IF  KTAPER  IS 1 A COSINE TAPER IS
C              APPLIED TO THE FIRST AND LAST TEN PER CENT OF  X  AND
C              THE FAST FOURIER TRANSFORM OF EACH COMPONENT IS COM-
C              PUTED. IF KTAPER=2 A TRIANGULAR TAPER IS APPLIED TO THE
C              ENTIRE SECTION. DESIGNATE THE FOURIER TRANSFORMS OF ANY TWO
C              COMPONENT SERIES OF  X  BY  U(J) AND V(J), J=0,1,...N/2
C              THE CO-SPECTRUM, C, IS THE REAL PART OF THE PRODUCT
C              U(J)*V(J)  AND THE QUADRATURE SPECTRUM, Q, IS THE
C              IMAGINARY PART.  NEIGHBORING FREQUENCIES MAY BE COM-
C              BINED TO FORM TRIANGULAR AVERAGES.  (M-1) FREQUENCIES
C              ON BOTH SIDES ARE COMBINED WITH THE CENTRAL FREQUENCY
C              IN THE WEIGHTED AVERAGE.  M  MUST BE A POWER OF 2.  THE
C              CROSS SPECTRA ARE ADDED TO AND STORED AT  SP(I,J,K).
C              NOT ALL HARMONICS NEED BE STORED.
C
C
C LIBRARY ROUTINES USED -  FFTTWO
C
C                     ----USAGE----
C SAMPLE CALL
C     P = PCROSP(X,NN,NK,KOFSET,KMAX,M,SP,SCRACH,CF,KTAPR,IE,NEWK)
C
C
C INPUTS
C
C     X(I,L)   AN ARRAY OF  NK  COMPONENT SERIES EACH OF LENGTH  NN
C              FOR WHICH THE CROSS SPECTRA ARE TO BE COMPUTED.
C
C     NN       THE 2ND DIMENSION OF ARRAY  X - THE NUMBER OF VALUES OF
C              EACH COMPONENT OF  X  IN ONE SECTION.  NN  MUST BE A
C              POWER OF 2.
C
C     NK       1ST DIMENSION OF ARRAY  X, WHICH IS THE NUMBER OF COM-
C              PONENT SERIES.  NK  IS ALSO THE 1ST AND 2ND DIMENSIONS
C              OF  SP.
C
C     KOFSET   SP(I,J,1)  WILL CONTAIN THE CROSS SPECTRAL VALUE COR-
C              RESPONDING TO HARMONIC  KOFSET+1
C
C     KMAX     THE 3RD DIMENSION OF  SP - THE NUMBER OF HARMONICS IN
C              SP.  KMAX  MUST BE .LE. NN/2+1
C
C     M        THE DIMENSION OF  CF  FOR DECIMATION IN FREQUENCY.
C              (M-1) HARMONICS ON EACH SIDE OF THE SPECTRAL VALUE AT
C              KOFSET+1, KOFSET+1+M, KOFSET+1+2*M,...
C              KOFSET+1+(KMAX-1)*M  ARE COMBINED WITH THE CENTRAL
C              FREQUENCY TO FORM TRIANGULAR AVERAGES.  M  MUST BE A
C              POWER OF 2
C
C     SCRACH(K)  A SCRATCH OR TEMPORARY ARRAY OF DIMENSION NN
C
C     CF(I)    AN ARRAY OF DIMENSION (M-1) IN WHICH  PCROSP  STORES
C              (M-I)/M  FOR TRIANGULAR AVERAGE
C
C     KTAPER   =1  CAUSES A COSINE TAPER TO BE APPLIED TO THE 1ST AND
C              LAST TEN PER CENT OF  X.  IF  KTAPER=0 THEN THE TAPER
C              IS NOT APPLIED.
C              =2 APPLIES A TRIANGULAR TAPER TO EACH SECTION.
C
C     IE       = 0  IF THE SECTION IS NOT THE 1ST SECTION AND IS NOT
C              THE LAST SECTION
C              = 1  IF THE SECTION IS THE 1ST SECTION AND IS NOT THE
C              LAST SECTION
C              = 2  IF THE SECTION IS THE LAST SECTION AND IS NOT THE
C              1ST SECTION
C              = 3  IF THE SECTION IS THE 1ST SECTION AND IS THE LAST
C              SECTION
C
C     NEWK     IS USED WHEN DIFFERENT SETS OF CONSECUTIVE HARMONICS
C              ARE DESIRED, E. G. TO STORE HARMONICS 1 THROUGH 10 IN
C              SP1 AND HARMONICS 119 THROUGH 128 IN SP2, PCROSP SHOULD
C              BE CALLED TWICE PER SECTION WITH NEWK=0, KOFSET=0,
C              KMAX=10  IN THE 1ST CALL AND NEWK=1, KOFSET=118,
C              KMAX=10  IN THE 2ND.  IF  NEWK .NE. 0  THE FFT IS NOT
C              COMPUTED.  NN AND NK  MUST BE THE SAME AS FOR THE
C              PREVIOUS CALL BUT  M  MAY BE DIFFERENT.
C
C
C OUTPUTS
C
C     SP(I,J,K)  SP(I,I,K)  CONTAINS THE POWER SPECTRUM FOR COMPONENT
C              I  OF  X.  SP(J,I,K), J .GT. I, CONTAINS THE CO-
C              SPECTRUM AND S(I,J,K) CONTAINS THE QUADRATURE SPECTRUM
C              FOR HARMONICS  KOFSET+1, KOFSET+1+M, KOFSET+1+2*M,...
C
C     P        = 0  NO ERRORS
C              = 1  NN IS NOT A POWER OF 2
C              = 2  KMAX .GT. 1+NN/2
C              = 3  NK .LE. 0
C              = 4  M .LE. 0
C              = 5  KOFSET .LT. -1  OR  KOFSET .GT. NN/2
C              = 6  SECTION LENGTH NOT SAME FOR ALL SECTIONS OR FOR
C              NEW K
C              = 7  NUMBER OF COMPONENTS NOT SAME FOR ALL SECTIONS OR
C              FOR NEW K
C              = 8  TAPER SWITCH NOT SAME FOR ALL SECTIONS OR FOR NEWK
C
C
      DIMENSION X(NN,NK),SP(KMAX,NK,NK),SCRACH(NN),CF(M)
      COMMON /OWNPCR/ NP,NH,QN,N,NC,KTAPER,NP1
      DATA PI /3.141592653/
      DATA TWOPI /6.2831853072/
      N = NN
      PCROSP = 2.
      IF (KMAX .GT. N/2+1) RETURN
      IF (IE .NE. 1 .AND. IE .NE. 3) GO TO 1600
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                    C
C                  INITIALIZE FOR FIRST SECTION
C                                                                    C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      PCROSP = 3.
      IF (NK .LE. 0) RETURN
      PCROSP = 4.
      IF (M .LE. 0) RETURN
      PCROSP = 5.
      IF (KOFSET .LT. -1 .OR. KOFSET .GT. N/2) RETURN
      NC = NK
      KSIZE = KMAX*NC*NC
      DO 1100 K=1,NC
      DO 1100 KK=1,NC
      DO 1100 KKK=1,KMAX
 1100 SP(KKK,KK,K)=0.
      IF (NEWK .NE. 0) GO TO 1600
      NP = 0
      NH = N/2
      QN = 1./N
      NP1 = N + 1
      KTAPER = KTAPR
 1600 PCROSP = 6.
C        ERROR RETURN IF SECTION LENGTH NOT SAME FOR ALL SECTIONS
C        AND FOR NEW  K
      IF (N/2 .NE. NH) RETURN
      PCROSP = 7.
C        ERROR RETURN IF NUMBER OF COMPONENTS NOT SAME FOR ALL
C        SECTIONS AND NEW  K
      IF (NK .NE. NC) RETURN
      PCROSP = 8.
C        ERROR RETURN IF TAPER SWITCH NOT SAME FOR ALL SECTIONS AND
C        NEW  K
      IF (KTAPER .NE. KTAPR) RETURN
      PCROSP = 0.
      IF (NEWK .NE. 0) GO TO 1700
      NP = NP + 1
 1700 MM1 = M - 1
      QM = 1./M
      DO 1500 I=1,MM1
 1500 CF(I) = (M-I)*QM
      DO 4100 I=1,NC
      IF (NEWK .NE. 0) GO TO 3300
      XMEAN = 0.
      DO 2100 J=1,N
 2100 XMEAN = XMEAN + X(J,I)
      XMEAN = XMEAN * QN
      DO 2200 J=1,N
 2200 X(J,I) = (X(J,I) - XMEAN) * QN
      IF (KTAPER .EQ. 0) GO TO 2500
C        APPLY COSINE TAPER TO 1ST AND LAST 10 PER C+NT OF SECTION
      NT = N/10
      IF(KTAPER.EQ.2)NT=N/2
      NMJ=N
      DO 2400 J=1,NT
      IF(KTAPER.EQ.1) W = .5 * (1. - COS(PI*(J-1)/NT))
      IF(KTAPER.EQ.2)W=J
      X(J,I) = W * X(J,I)
      X(NMJ,I) = W * X(NMJ,I)
 2400 NMJ = NMJ - 1
C
C        COMPUTE FAST FOUTIER TRANSFORM OF COMPLEX SERIES,
C        U(T)+I*V(T) = X(2T)+I*X(2T+1)  FOR T=0,...,N/2-1
C
 2500 CALL FFT(X(1,I),X(2,I),NH,NH,NH,2,PCROSP)
C
C        TRANSFORM OF  X, TX = TU + EXP(2PI*I*J/N) * TV
C
      A = .5*(X(1,I) - X(2,I))
      X(1,I) = .5*(X(1,I) + X(2,I))
      X(2,I) = A
      NHMJ = N-1
      COST = COS(TWOPI/N)
      SINT = SIN(TWOPI/N)
      CS = COST
      SN = SINT
      DO 3100 J=1,NH,2
      A = .5 * (X(J+2,I) + X(NHMJ,I))
      B = .5 * (X(J+3,I) - X(NHMJ+1,I))
      C = .5 * (X(J+3,I) + X(NHMJ+1,I))
      D = .5 * (-X(J+2,I) + X(NHMJ,I))
      X(J+2,I) = A + C*CS - D*SN
      X(J+3,I) = B + C*SN + D*CS
      X(NHMJ,I) = A - C*CS + D*SN
      X(NHMJ+1,I) = -B + C*SN + D*CS
      A = CS*COST - SN*SINT
      SN = SN*COST + CS*SINT
      CS = A
 3100 NHMJ = NHMJ - 2
C        POWER SPECTRUM  SP(K,I,I) = ABS(TX)
C        X(J)  THROUGH  X(JE)  ARE USED TO COMPUTE  SCRACH(1)  THROUGH
C        SCRACH(KJE)
 3300 KJS = 1
      FLAG = 0.
      J = 2*(KOFSET-M+3) - 1
      JKS = M
      IF (J .GT. 1) GO TO 3400
      SCRACH(1) = X(1,I)*X(1,I)
      J = 3
      KJS = 2
      JKS = KOFSET + 2
 3400 JE = 2*(KOFSET + KMAX*M + 1) - 1
      IF (JE .LT. N) GO TO 3500
      FLAG = 1.
      JE = 2*NH - 1
      KJEP1 = KJS + (JE-J)/2 + 1
      SCRACH(KJEP1) = X(2,I)*X(2,I)
 3500 KJE = KJS + (JE-J)/2
      JS = J
      DO 3600 KJ=KJS,KJE
      SCRACH(KJ) = X(J,I)*X(J,I) + X(J+1,I)*X(J+1,I)
 3600 J = J + 2
C
C
C
C        FOR  NC  POWER SPECTRA, SUM SECTIONS FOR  KMAX  HARMONICS
C        STARTING WITH HARMONIC  KOFSET+1  AT INTERVAL  M  HARMONICS
      J = JKS
      DO 4100 K=1,KMAX
      SP(K,I,I) = SP(K,I,I) + SCRACH(J)
      IF (MM1 .LE. 0) GO TO 4100
C        COMPUTE TRIANGULAR AVERAGE OVER  M-1  NEIGHBORING HARMONICS
      DO 4000 IM=1,MM1
      JP = J + IM
      IF (JP .LT. NH+2) SP(K,I,I) = SP(K,I,I) + CF(IM)*SCRACH(JP)
      JM = J - IM
      IF (JM .GE. 0) SP(K,I,I) = SP(K,I,I) + CF(IM)*SCRACH(JM)
 4000 CONTINUE
 4100 J = J + M
CCCCC CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                    C
C        CROSS SPECTRUM S(XY) = C(XY) + I*Q(XY)                      C
C                            = TRANSFORM(X) * TRANSFORM(Y)           C
C                                                                    C
CCCCC CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      MX2 = M*2
      NCM1 = NC - 1
C        COMPUTE  (NC-1)(NC)/2  CROSS-SPECTRA
      KJS = 2*KJS - 1
      KJEP1 = 2*KJE + 1
      KJ = KJS
      DO 6100 I1 = 1,NCM1
      IP1 = I1 + 1
      DO 6100 I2=IP1,NC
      SCRACH(1) = X(1,I1)*X(1,I2)
      SCRACH(2) = 0.
      IF (FLAG .EQ. 0) GO TO 4500
      SCRACH(KJEP1) = X(2,I1)*X(2,I2)
      SCRACH(KJEP1 + 1) = 0.
 4500 KJS = KJ
      DO 5100 J=JS,JE,2
      A = X(J,I1)
      B = X(J+1,I1)
      C = X(J,I2)
      D = X(J+1,I2)
      SCRACH(KJS) = A*C + B*D
      SCRACH(KJS+1) = B*C - A*D
 5100 KJS = KJS + 2
C        SUM CROSS SPECTRA OVER SE3TIONS FOR  KMAX  HARMONICS STARTING
C        WITH HARMONIC  KOFSET+1  AT INTERVAL  M  HARMONICS
      J = 2*JKS - 1
      DO 6100 K=1,KMAX
      SP(K,I2,I1) = SP(K,I2,I1) + SCRACH(J)
      SP(K,I1,I2) = SP(K,I1,I2) + SCRACH(J+1)
      IF (MM1 .LE. 0) GO TO 6100
C        COMPUTE TRIANGULAR AVERAGE OVER  M-1  NEIGHBORING FREQUENCIES
      JP = J+2
      JM = J-2
      DO 6000 I=1,MM1
      A = SP(K,I2,I1)
      B = SP(K,I1,I2)
      C = CF(I)
      IF (JP .GT. NP1) GO TO 5500
      A = A + C*SCRACH(JP)
      B = B + C*SCRACH(JP+1)
 5500 IF (JM .LT. 1) GO TO 5900
      A = A + C*SCRACH(JM)
      B = B + C*SCRACH(JM+1)
 5900 SP(K,I2,I1) = A
      SP(K,I1,I2) = B
      JP = JP + 2
 6000 JM = JM - 2
 6100 J = J + MX2
      IF (IE .LT. 2) RETURN
CCCCC CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                    C
C        IF FINAL SECTION, MULTIPLY BY  2/NP  FOR AVERAGE OVER       C
C        SECTIONS AND TRANSPOSE TO GET CROSS-SPECTRAL MATRIX         C
C        STORED BY ROWS ONE FREQUENCY AT A TIME                      C
C                                                                    C
CCCCC CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      QNP2 = 2./NP
      DO 6500 K=1,NC
      DO 6500 KK=1,NC
      DO 6500 KKK=1,KMAX
 6500 SP(KKK,KK,K)=SP(KKK,KK,K)*QNP2
      RETURN
      END

