      subroutine desl1m(x,y,n,f,wt,ind,a,b,iter)
c
c   Finds the best fitting line, y = a*x + b, to the n points in the
c    arrays x and y, according to the L1 criterion
c
c   The arrays f, wt, and ind are working space; f is the array of
c    slope values between points (from a particular point xi), and
c    wt the differences between all the x's and xi; both of these,
c    along with the indexing array ind, are used in finding the
c    weighted median
c
c   iter is the iteration count, but is also used on input to flag
c    whether initial values of a and b are input (iter >= 0) or
c    computed (iter < 0 will use least-squares; iter=0 will use
c    original J&S code). If the number of iterations exceeds the number
c    of data, the program will assume that there it is not converging
c    and return, with iter set negative.
c   
c   Uses a descent algorithm. Follows program in L. A. Josvanger
c    and V. A. Sposito, "L1 norm estimates and the simple regression
c    problem", Commun. Stat., vol 12, pp 215-221 (1982), with the
c    initialization suggested in R. W. Farebrother, "Least
c    squares initial values for the L1-norm fitting of a straight line",
c    Appl. Stat., vol 41, pp. 627-633 (1992).
c
c   This version by D. C. Agnew, July 2004
c
      dimension x(*),y(*),wt(*),ind(*),f(*)
      data tol/1.e-6/
      xn = n
      iflg = 0
      do i=2,n
        if(x(i).ne.x(1)) iflg = 1
      enddo
c check for unfittable case (all x's the same)
      if(iflg.eq.0) then
        iter = 0
        return
      endif
c
c  find initial values for a and b
c
      if(iter.eq.0) then
        sy = 0.0
        do i=1,n
          sy = sy + y(i)
        enddo
        b = 0.
        a = sy/n
      endif
      if(iter.lt.0) then
        sx=0
        sy=0
        sxx=0
        sxy=0
        do i=1,n
          sx=sx+x(i)
          sxx=sxx+x(i)*x(i)
          sy=sy+y(i)
          sxy=sxy+y(i)*x(i)
        enddo
        d=n*sxx-sx*sx
        a=(sxx*sy - sx*sxy)/d
        b=(n*sxy - sx*sy)/d
      endif
c
c  set the starting point to be the one with minimum deviation
c
      iresa = 1
      iresb = 0
      dev = abs(y(1) - a - b*x(1))
      do i=2,n
         dt=abs(y(i) - a -b*x(i))
         if(dt.le.dev) then
           dev = dt
           iresa = i
         endif
      enddo
      iter = 0
c
c  save x and y that correspond to minimum absolute deviation
c
 30   j = iresa
      xj = x(j)
      yj = y(j)
      i = 1
      m = 1
      k = n
      twu = 0.
      twl = 0.
c
c  separate slope values greater than b from those less than b
c
 40   xij = x(i) - x(j)
      if(xij.eq.0) go to 60
      fi = (y(i) - yj)/xij
      if(xij.lt.0) xij = -xij
      if(fi.gt.b) go to 50
      f(m) = fi
      wt(m) = xij
      ind(m) = i
      twl = twl + xij
      m = m + 1
      go to 60
 50   f(k) = fi
      wt(k) = xij
      ind(k) = i
      twu = twu + xij
      k = k - 1
 60   if(i.eq.n) go to 70
      i = i + 1
      go to 40
c
c  set the new value of b to be the weight median slope value
c
 70   asum = (twl + twu)/2.
      if(twl.ge.twu) go to 130
      m = m - 1
 80   k = k + 1
      m = m + 1
      i = k
 90   fnew = f(i)
      inew = i
 100  if(i.eq.n) go to 110
      i = i + 1
      if(f(i).lt.fnew) go to 90
      go to 100
 110  twl = twl + wt(inew)
      if(twl.ge.asum) go to 120
      f(inew) = f(k)
      wt(inew) = wt(k)
      ind(inew) = ind(k)
      go to 80
 120  bnew = fnew
      jnew = ind(inew)
      go to 180
 130  m = m - 1
      i = m
 140  fnew = f(i)
      inew = i
 150  if(i.eq.1) go to 160
      i = i - 1
      if(f(i).gt.fnew) go to 140
      go to 150
 160  twu = twu + wt(inew)
      if(twu.gt.asum) go to 170
      f(inew) = f(m)
      wt(inew) = wt(m)
      ind(inew) = ind(m)
      go to 130
 170  bnew = fnew
      jnew = ind(inew)
c
c  find new intercept value
c
 180  a = yj - bnew*xj
      iter = iter + 1
      if(iter.gt.n) then
        iter=-iter
        return
      endif
c
c  test to see if b values are changing
c    
      if(abs(bnew-b).le.tol) go to 190
      b = bnew
c
c  test to see if index pattern is repeating
c
      if(iresb.eq.jnew) go to 190
      iresb = iresa
      iresa = jnew
      go to 30
 190  return
      end
